#include "bsp.h"
#include "mrfi.h"
#include "bsp_leds.h"
#include "bsp_buttons.h"
#include "nwk_types.h"
#include "nwk_api.h"
#include "nwk_frame.h"
#include "nwk.h"
#include "msp430x22x4.h"
#include "vlo_rand.h"

#define MESSAGE_LENGTH 3
   char msg[20];
void TXString( char* string, int length );
void MCU_Init(void);
void transmitData(int addr, signed char rssi,  char msg[MESSAGE_LENGTH] );
void transmitDataString(char addr[4],char rssi[3], char msg[MESSAGE_LENGTH]);
void createRandomAddress();
__no_init volatile int tempOffset @ 0x10F4; // Temperature offset set at production
__no_init volatile char Flash_Addr[4] @ 0x10F0; // Flash address set randomly

// reserve space for the maximum possible peer Link IDs
static linkID_t sLID[NUM_CONNECTIONS];
// callback handler
static uint8_t sCB(linkID_t);
// work loop semaphores
static uint8_t sPeerFrameSem;
static uint8_t sJoinSem;
static uint8_t sSelfMeasureSem;

// mode data verbose = default, deg F = default
char verboseMode = 0;
char degCMode = 1;

typedef struct
   {
   unsigned char number;
   unsigned char type;
   unsigned char measurement;
   } perifery;

struct 
   {
   unsigned char number;
   perifery member[10];
   } periferies;

unsigned char rx_buffer[64];
unsigned char rx_queue_size;

void main (void)
   {
   addr_t lAddr;
   bspIState_t intState;
   WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
      {
      // delay loop to ensure proper startup before SimpliciTI increases DCO
      // This is typically tailored to the power supply used, and in this case
      // is overkill for safety due to wide distribution.
      volatile int i;
      for(i = 0; i < 0xFFFF; i++){}
      }
   if( CALBC1_8MHZ == 0xFF )                 // Do not run if cal values are erased
      {
      volatile int i;
      P1DIR |= 0x03;      BSP_TURN_ON_LED1();      BSP_TURN_OFF_LED2();
      while(1)
         {
         for(i = 0; i < 0x5FFF; i++){}
         BSP_TOGGLE_LED2();BSP_TOGGLE_LED1();
      }  }
   BSP_Init();
   if( Flash_Addr[0] == 0xFF && Flash_Addr[1] == 0xFF && Flash_Addr[2] == 0xFF && Flash_Addr[3] == 0xFF )
      {
      createRandomAddress();                  // set Random device address at initial startup
      }
   lAddr.addr[0]=Flash_Addr[0];   lAddr.addr[1]=Flash_Addr[1];   lAddr.addr[2]=Flash_Addr[2];   lAddr.addr[3]=Flash_Addr[3];
   
   SMPL_Ioctl(IOCTL_OBJ_ADDR, IOCTL_ACT_SET, &lAddr);
   MCU_Init();
   SMPL_Init(sCB);
  
    
   //MRFI_SetRxAddrFilter(lAddr.addr);
   //MRFI_EnableRxAddrFilter();
   // main work loop
   while (1)
      {
      if (rx_queue_size >= 2) 
         {
         if (rx_buffer[0]=='?') 
            {
            msg[0]=periferies.number;
            TXString(msg,1);
            }
         else
            {
            SMPL_Send(sLID[rx_buffer[0]], &rx_buffer[1], 1);
            //SMPL_Send(sLID[0], &rx_buffer[1], 1);
            TXString("!",1);
            }
  //       rx_buffer[0]=rx_buffer[1];
  //       rx_buffer[1]=rx_buffer[2];
  //       rx_buffer[2]=rx_buffer[3];
  //       rx_buffer[3]=rx_buffer[4];
  //       rx_buffer[4]=rx_buffer[5];
  //       rx_buffer[5]=rx_buffer[6];
  //       rx_buffer[6]=0;
         rx_queue_size-=2;         
         }
 
        
        // Wait for the Join semaphore to be set by the receipt of a Join frame from a
      // device that supports and End Device.
    
      if (sJoinSem && (periferies.number < NUM_CONNECTIONS))
         {
         SMPL_LinkListen(&sLID[periferies.number]);
         periferies.number++;
         BSP_ENTER_CRITICAL_SECTION(intState);
         if (sJoinSem) sJoinSem--;
         BSP_EXIT_CRITICAL_SECTION(intState);
//         TXString("prijavio se",11);
         }
    

       if (sPeerFrameSem)
          {
          uint8_t     msg[2], len;
          SMPL_Receive(sLID[rx_buffer[0]], msg, &len);
//          SMPL_Receive(sLID[0], msg, &len);
          TXString(msg,len);
          BSP_ENTER_CRITICAL_SECTION(intState);
          sPeerFrameSem--;
          BSP_EXIT_CRITICAL_SECTION(intState);
          }        
   }  }

/*------------------------------------------------------------------------------
*
------------------------------------------------------------------------------*/
void createRandomAddress()
   {
   unsigned int rand, rand2;
   do
      {
      rand = TI_getRandomIntegerFromVLO();    // first byte can not be 0x00 of 0xFF
      } while( (rand & 0xFF00)==0xFF00 || (rand & 0xFF00)==0x0000 );
   rand2 = TI_getRandomIntegerFromVLO();
  
   BCSCTL1 = CALBC1_1MHZ;                    // Set DCO to 1MHz
   DCOCTL = CALDCO_1MHZ;
   FCTL2 = FWKEY + FSSEL0 + FN1;             // MCLK/3 for Flash Timing Generator
   FCTL3 = FWKEY + LOCKA;                    // Clear LOCK & LOCKA bits
   FCTL1 = FWKEY + WRT;                      // Set WRT bit for write operation
  
   Flash_Addr[0]=(rand>>8) & 0xFF;
   Flash_Addr[1]=rand & 0xFF;
   Flash_Addr[2]=(rand2>>8) & 0xFF; 
   Flash_Addr[3]=rand2 & 0xFF; 
  
   FCTL1 = FWKEY;                            // Clear WRT bit
   FCTL3 = FWKEY + LOCKA + LOCK;             // Set LOCK & LOCKA bit
   }

void TXString( char* string, int length )
   {
   int pointer;
   for( pointer = 0; pointer < length; pointer++)
      {
      volatile int i;
      while (!(IFG2&UCA0TXIFG));
      UCA0TXBUF = string[pointer];
      while (!(IFG2&UCA0TXIFG));              // USCI_A0 TX buffer ready?
   }  }


void MCU_Init()
   {
   BCSCTL1 = CALBC1_8MHZ;                    // Set DCO
   DCOCTL = CALDCO_8MHZ;
  
   BCSCTL3 |= LFXT1S_2;                      // LFXT1 = VLO
   TACCTL0 = CCIE;                           // TACCR0 interrupt enabled
   TACCR0 = 12000;                           // ~1 second
   TACTL = TASSEL_1 + MC_1;                  // ACLK, upmode
  
   P3SEL |= 0x30;                            // P3.4,5 = USCI_A0 TXD/RXD
   UCA0CTL1 = UCSSEL_2;                      // SMCLK
   UCA0BR0 = 0x41;                           // 9600 from 8Mhz
   UCA0BR1 = 0x3;
   UCA0MCTL = UCBRS_2;                       
   UCA0CTL1 &= ~UCSWRST;                     // **Initialize USCI state machine**
   IE2 |= UCA0RXIE;                          // Enable USCI_A0 RX interrupt
   __enable_interrupt(); 
   

   
   }

/*------------------------------------------------------------------------------
* Runs in ISR context. Reading the frame should be done in the
* application thread not in the ISR thread.
------------------------------------------------------------------------------*/
static uint8_t sCB(linkID_t lid)
   {
   linkID_t a = lid;
   if (a) 
     sPeerFrameSem++;
   else sJoinSem++;
   return 0;
   }

/*------------------------------------------------------------------------------
* ADC10 interrupt service routine
------------------------------------------------------------------------------*/
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR(void)
   {
    __bic_SR_register_on_exit(CPUOFF);        // Clear CPUOFF bit from 0(SR)
   }

/*------------------------------------------------------------------------------
* Timer A0 interrupt service routine
------------------------------------------------------------------------------*/
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A (void)
   {
   sSelfMeasureSem = 1;
   }

/*------------------------------------------------------------------------------
* USCIA interrupt service routine
------------------------------------------------------------------------------*/
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void)
   {
   //char periferiesCounter;

   char rx;
   rx=UCA0RXBUF;
   rx_buffer[rx_queue_size++] = rx;   
   }
